<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Pinpoint\CarouselManager\Block\Adminhtml\Carousel\Edit;

use Magento\Backend\Block\Widget\Context;
use Magento\Framework\View\Element\UiComponent\Control\ButtonProviderInterface;
use Pinpoint\CarouselManager\Helper\Config;

class AddSlideButton extends GenericButton implements ButtonProviderInterface
{
    /**
     * @var Config
     */
    private $configHelper;
    /**
     * @var \Magento\Framework\Registry
     */
    private $registry;

    public function __construct(Context $context, Config $configHelper, \Magento\Framework\Registry $registry)
    {
        parent::__construct($context);
        $this->configHelper = $configHelper;
        $this->registry = $registry;
    }

    /**
     * @return array
     */
    public function getButtonData()
    {
        $data = [];
        if ($this->getModelId() && $this->canAddMoreSlides()) {
            $data = [
                'label' => __('Add New Slide'),
                'class' => 'delete',
                'on_click' => "document.getElementsByClassName('add-slide-button')[0].click();",
                'sort_order' => 10,
            ];
        }
        return $data;
    }

    private function canAddMoreSlides()
    {
        $carousel = $this->registry->registry('pinpoint_carouselmanager_carousel');
        if (!is_object($carousel) || !$carousel->getId()) {
            return false;
        }

        $slidesCount = count($this->registry->registry('pinpoint_carouselmanager_carousel_slides'));
        return $slidesCount < $this->configHelper->getMaximumNumberOfSlides();
    }
}
