<?php
namespace Pinpoint\CarouselManager\Block;

use Magento\Catalog\Model\Template\Filter\Factory;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Pinpoint\CarouselManager\Helper\Config;
use Pinpoint\CarouselManager\Model\CarouselRepository;
use Pinpoint\CarouselManager\Model\SlideCtaRepository;
use Pinpoint\CarouselManager\Model\SlideRepository;

/**
 * Class Carousel
 * @package Pinpoint\CarouselManager\Block
 */
class Carousel extends Template
{
    const DEFAULT_COLOR_CODE = '#0000000';
    /**
     * @var
     */
    protected $carouselId;

    /**
     * @var CarouselRepository
     */
    protected $carouselRespository;
    /**
     * @var
     */
    protected $carousel;

    /**
     * @var string
     */
    protected $_template;

    /**
     * @var string
     */
    protected $templateFilterModel = 'Magento\Catalog\Model\Template\Filter';
    /**
     * @var
     */
    protected $templateProcessor;
    /**
     * @var Factory
     */
    protected $templateFilterFactory;

    /**
     * @var array
     */
    protected $slides = [];

    /**
     * @var SlideRepository
     */
    private $slideRepository;
    /**
     * @var \Magento\Backend\Model\UrlInterface
     */
    private $urlBuilder;
    /**
     * @var Config
     */
    private $configHelper;
    /**
     * @var Json
     */
    private $json;
    /**
     * @var SlideCtaRepository
     */
    private $slideCtaRepository;

    /**
     * Carousel constructor.
     * @param CarouselRepository $carouselRepository
     * @param Factory $templateFilterFactory
     * @param FilterProvider $filterProvider
     * @param Context $context
     * @param \Magento\Backend\Model\UrlInterface $urlBuilder
     * @param Config $configHelper
     * @param Json $json
     * @param SlideCtaRepository $slideCtaRepository
     * @param array $data
     */
    public function __construct(
        CarouselRepository $carouselRepository,
        SlideRepository $slideRepository,
        Factory $templateFilterFactory,
        Context $context,
        \Magento\Backend\Model\UrlInterface $urlBuilder,
        Config $configHelper,
        Json $json,
        SlideCtaRepository $slideCtaRepository,
        array $data = []
    )
    {
        $this->templateFilterFactory = $templateFilterFactory;
        $this->carouselRespository = $carouselRepository;
        $this->slideRepository = $slideRepository;
        $this->_template = 'Pinpoint_CarouselManager::carousel.phtml';
        $this->urlBuilder = $urlBuilder;
        $this->configHelper = $configHelper;
        $this->json = $json;
        $this->slideCtaRepository = $slideCtaRepository;

        if (isset($data['template']) && $data['template']) {
            $this->setTemplate($data['template']);
        }

        if (isset($data['carousel_id']) && $data['carousel_id']) {
            $this->setCarouselId($data['carousel_id']);
        }

        parent::__construct($context, $data);
    }

    /**
     * @return mixed
     */
    public function getCarouselId()
    {
        return $this->carouselId;
    }

    /**
     * @param $carouselId
     */
    public function setCarouselId($carouselId)
    {
        $this->carouselId = $carouselId;
    }

    /**
     * @return mixed
     */
    public function getCurrentStore()
    {
        return $this->_storeManager->getStore()->getId();
    }

    /**
     * @return \Pinpoint\CarouselManager\Model\Carousel|null
     */
    public function getCarousel()
    {
        if (!$this->carousel) {

            try {
                $carouselId = $this->getCarouselId();
                if(!is_null($carouselId)) {
                    $carousel = $this->carouselRespository->getById($carouselId);
                } else {
                    throw new \Exception('Impossible to generate a carousel. Please set up a carousel ID.');
                }
            } catch (\Exception $e) {
                // Silently skip
                $this->_logger->critical($e);
                return null;
            }

            $this->setCarousel($carousel);
        }

        return $this->carousel;
    }

    public function getSlides()
    {
        return $this->slideRepository->getByCarouselId($this->carouselId);
    }

    public function getSlideCtas($slide)
    {
        return $this->slideCtaRepository->getBySlideId($slide->getId());
    }

    /**
     * @return string
     */
    protected function _toHtml()
    {
        if (!$this->getCarousel()) {
            return '';
        }
        return parent::_toHtml();
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _getTemplateProcessor()
    {
        if (null === $this->templateProcessor) {
            $this->templateProcessor = $this->templateFilterFactory->create($this->templateFilterModel);
        }

        return $this->templateProcessor;
    }

    /**
     * @param $attribute
     * @return bool
     */
    public function isFilterable($attribute)
    {
        return in_array($attribute, ['title', 'description', 'cta', 'link']);
    }

    /**
     * @param $slide
     * @param $attributeImage
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getImageUrl($filename)
    {
        if (!$filename) {
            return '';
        }

        return $this->urlBuilder->getBaseUrl(['_type' =>\Magento\Framework\UrlInterface::URL_TYPE_MEDIA]) . 'content/carousel/' . $filename;
    }

    /**
     * @return bool|false|string
     */
    public function getCarouselConfig()
    {
        $carousel = $this->getCarousel();
        $config = [
            'name' => $carousel->getName(),
            'identifier' => $carousel->getIdentifier(),
            'fade' => $carousel->getTransitionType() == '1',
            'transition_speed' => $carousel->getTransitionSpeed(),
        ];

        return $this->json->serialize($config);
    }

    /**
     * @param $carousel
     */
    public function setCarousel($carousel)
    {
        $currentStoreId = $this->getCurrentStore();
        $allowStores = $carousel->getStoreIds();

        if ($allowStores && $currentStoreId &&
            is_array($allowStores) &&
            in_array($currentStoreId, $allowStores)
        ) {
            $this->carousel = $carousel;
        }
    }

    public function getColourCode($colourCode)
    {
        if (!$colourCode) {
            $colourCode = self::DEFAULT_COLOR_CODE;
        }

        return $colourCode;
    }
}
